"""
wagtail-localize-dashboard
~~~~~~~~~~~~~~~~~~~~~~~~~~~

A translation dashboard for Wagtail sites using wagtail-localize.
Provides cached translation progress tracking with automatic updates.

:copyright: (c) 2025 Lincoln Loop
:license: MIT, see LICENSE for more details.
"""

__version__ = "0.3.0"
__author__ = "Lincoln Loop"
__license__ = "MIT"

default_app_config = "wagtail_localize_dashboard.apps.DashboardConfig"
